@@ -5,7 +5,10 @@
 #![deny(unused_mut)]
 #![deny(broken_intra_doc_links)]
 
+#![cfg_attr(docsrs, feature(doc_auto_cfg))]
+
 #![cfg_attr(feature = "strict", deny(warnings))]
+#![cfg_attr(all(not(feature = "std"), not(test)), no_std)]
 
 //! This crate provides data structures to represent
 //! [lightning BOLT11](https://github.com/lightningnetwork/lightning-rfc/blob/master/11-payment-encoding.md)
@@ -15,6 +18,10 @@
 //!   * For parsing use `str::parse::<Invoice>(&self)` (see the docs of `impl FromStr for Invoice`)
 //!   * For constructing invoices use the `InvoiceBuilder`
 //!   * For serializing invoices use the `Display`/`ToString` traits
+
+#[cfg(not(any(feature = "std", feature = "no-std")))]
+compile_error!("at least one of the `std` or `no-std` features must be enabled");
+
 pub mod payment;
 pub mod utils;
 
@@ -23,6 +30,12 @@ extern crate bitcoin_hashes;
 #[macro_use] extern crate lightning;
 extern crate num_traits;
 extern crate secp256k1;
+extern crate alloc;
+#[cfg(any(test, feature = "std"))]
+extern crate core;
+
+#[cfg(feature = "std")]
+use std::time::SystemTime;
 
 use bech32::u5;
 use bitcoin_hashes::Hash;
@@ -32,35 +45,60 @@ use lightning::ln::features::InvoiceFeatures;
 #[cfg(any(doc, test))]
 use lightning::routing::network_graph::RoutingFees;
 use lightning::routing::router::RouteHint;
+use lightning::util::invoice::construct_invoice_preimage;
 
 use secp256k1::key::PublicKey;
 use secp256k1::{Message, Secp256k1};
 use secp256k1::recovery::RecoverableSignature;
 
-use std::fmt::{Display, Formatter, self};
-use std::iter::FilterMap;
-use std::ops::Deref;
-use std::slice::Iter;
-use std::time::{SystemTime, Duration, UNIX_EPOCH};
+use core::fmt::{Display, Formatter, self};
+use core::iter::FilterMap;
+use core::ops::Deref;
+use core::slice::Iter;
+use core::time::Duration;
 
 mod de;
 mod ser;
 mod tb;
 
+mod prelude {
+	#[cfg(feature = "hashbrown")]
+	extern crate hashbrown;
+
+	pub use alloc::{vec, vec::Vec, string::String, collections::VecDeque, boxed::Box};
+	#[cfg(not(feature = "hashbrown"))]
+	pub use std::collections::{HashMap, HashSet, hash_map};
+	#[cfg(feature = "hashbrown")]
+	pub use self::hashbrown::{HashMap, HashSet, hash_map};
+
+	pub use alloc::string::ToString;
+}
+
+use prelude::*;
+
+/// Sync compat for std/no_std
+#[cfg(feature = "std")]
+mod sync {
+	pub use ::std::sync::{Mutex, MutexGuard};
+}
+
+/// Sync compat for std/no_std
+#[cfg(not(feature = "std"))]
+mod sync;
+
 pub use de::{ParseError, ParseOrSemanticError};
 
-// TODO: fix before 2037 (see rust PR #55527)
-/// Defines the maximum UNIX timestamp that can be represented as `SystemTime`. This is checked by
-/// one of the unit tests, please run them.
-const SYSTEM_TIME_MAX_UNIX_TIMESTAMP: u64 = std::i32::MAX as u64;
+/// The number of bits used to represent timestamps as defined in BOLT 11.
+const TIMESTAMP_BITS: usize = 35;
 
-/// Allow the expiry time to be up to one year. Since this reduces the range of possible timestamps
-/// it should be rather low as long as we still have to support 32bit time representations
-const MAX_EXPIRY_TIME: u64 = 60 * 60 * 24 * 356;
+/// The maximum timestamp as [`Duration::as_secs`] since the Unix epoch allowed by [`BOLT 11`].
+///
+/// [BOLT 11]: https://github.com/lightning/bolts/blob/master/11-payment-encoding.md
+pub const MAX_TIMESTAMP: u64 = (1 << TIMESTAMP_BITS) - 1;
 
 /// Default expiry time as defined by [BOLT 11].
 ///
-/// [BOLT 11]: https://github.com/lightningnetwork/lightning-rfc/blob/master/11-payment-encoding.md
+/// [BOLT 11]: https://github.com/lightning/bolts/blob/master/11-payment-encoding.md
 pub const DEFAULT_EXPIRY_TIME: u64 = 3600;
 
 /// Default minimum final CLTV expiry as defined by [BOLT 11].
@@ -68,61 +106,10 @@ pub const DEFAULT_EXPIRY_TIME: u64 = 3600;
 /// Note that this is *not* the same value as rust-lightning's minimum CLTV expiry, which is
 /// provided in [`MIN_FINAL_CLTV_EXPIRY`].
 ///
-/// [BOLT 11]: https://github.com/lightningnetwork/lightning-rfc/blob/master/11-payment-encoding.md
+/// [BOLT 11]: https://github.com/lightning/bolts/blob/master/11-payment-encoding.md
 /// [`MIN_FINAL_CLTV_EXPIRY`]: lightning::ln::channelmanager::MIN_FINAL_CLTV_EXPIRY
 pub const DEFAULT_MIN_FINAL_CLTV_EXPIRY: u64 = 18;
 
-/// This function is used as a static assert for the size of `SystemTime`. If the crate fails to
-/// compile due to it this indicates that your system uses unexpected bounds for `SystemTime`. You
-/// can remove this functions and run the test `test_system_time_bounds_assumptions`. In any case,
-/// please open an issue. If all tests pass you should be able to use this library safely by just
-/// removing this function till we patch it accordingly.
-fn __system_time_size_check() {
-	// Use 2 * sizeof(u64) as expected size since the expected underlying implementation is storing
-	// a `Duration` since `SystemTime::UNIX_EPOCH`.
-	unsafe { std::mem::transmute_copy::<SystemTime, [u8; 16]>(&UNIX_EPOCH); }
-}
-
-
-/// **Call this function on startup to ensure that all assumptions about the platform are valid.**
-///
-/// Unfortunately we have to make assumptions about the upper bounds of the `SystemTime` type on
-/// your platform which we can't fully verify at compile time and which isn't part of it's contract.
-/// To our best knowledge our assumptions hold for all platforms officially supported by rust, but
-/// since this check is fast we recommend to do it anyway.
-///
-/// If this function fails this is considered a bug. Please open an issue describing your
-/// platform and stating your current system time.
-///
-/// # Panics
-/// If the check fails this function panics. By calling this function on startup you ensure that
-/// this wont happen at an arbitrary later point in time.
-pub fn check_platform() {
-    // The upper and lower bounds of `SystemTime` are not part of its public contract and are
-    // platform specific. That's why we have to test if our assumptions regarding these bounds
-    // hold on the target platform.
-    //
-    // If this test fails on your platform, please don't use the library and open an issue
-    // instead so we can resolve the situation. Currently this library is tested on:
-    //   * Linux (64bit)
-    let fail_date = UNIX_EPOCH + Duration::from_secs(SYSTEM_TIME_MAX_UNIX_TIMESTAMP);
-    let year = Duration::from_secs(60 * 60 * 24 * 365);
-
-    // Make sure that the library will keep working for another year
-    assert!(fail_date.duration_since(SystemTime::now()).unwrap() > year);
-
-    let max_ts = PositiveTimestamp::from_unix_timestamp(
-        SYSTEM_TIME_MAX_UNIX_TIMESTAMP - MAX_EXPIRY_TIME
-    ).unwrap();
-    let max_exp = ::ExpiryTime::from_seconds(MAX_EXPIRY_TIME).unwrap();
-
-    assert_eq!(
-        (*max_ts.as_time() + *max_exp.as_duration()).duration_since(UNIX_EPOCH).unwrap().as_secs(),
-        SYSTEM_TIME_MAX_UNIX_TIMESTAMP
-    );
-}
-
-
 /// Builder for `Invoice`s. It's the most convenient and advised way to use this library. It ensures
 /// that only a semantically and syntactically correct Invoice can be built using it.
 ///
@@ -142,6 +129,9 @@ pub fn check_platform() {
 ///
 /// use lightning_invoice::{Currency, InvoiceBuilder};
 ///
+/// # #[cfg(not(feature = "std"))]
+/// # fn main() {}
+/// # #[cfg(feature = "std")]
 /// # fn main() {
 /// let private_key = SecretKey::from_slice(
 ///		&[
@@ -186,11 +176,11 @@ pub struct InvoiceBuilder<D: tb::Bool, H: tb::Bool, T: tb::Bool, C: tb::Bool, S:
 	tagged_fields: Vec<TaggedField>,
 	error: Option<CreationError>,
 
-	phantom_d: std::marker::PhantomData<D>,
-	phantom_h: std::marker::PhantomData<H>,
-	phantom_t: std::marker::PhantomData<T>,
-	phantom_c: std::marker::PhantomData<C>,
-	phantom_s: std::marker::PhantomData<S>,
+	phantom_d: core::marker::PhantomData<D>,
+	phantom_h: core::marker::PhantomData<H>,
+	phantom_t: core::marker::PhantomData<T>,
+	phantom_c: core::marker::PhantomData<C>,
+	phantom_s: core::marker::PhantomData<S>,
 }
 
 /// Represents a syntactically and semantically correct lightning BOLT11 invoice.
@@ -280,14 +270,14 @@ pub struct RawDataPart {
 	pub tagged_fields: Vec<RawTaggedField>,
 }
 
-/// A timestamp that refers to a date after 1 January 1970 which means its representation as UNIX
-/// timestamp is positive.
+/// A timestamp that refers to a date after 1 January 1970.
 ///
 /// # Invariants
-/// The UNIX timestamp representing the stored time has to be positive and small enough so that
-/// a `EpiryTime` can be added to it without an overflow.
+///
+/// The Unix timestamp representing the stored time has to be positive and no greater than
+/// [`MAX_TIMESTAMP`].
 #[derive(Eq, PartialEq, Debug, Clone)]
-pub struct PositiveTimestamp(SystemTime);
+pub struct PositiveTimestamp(Duration);
 
 /// SI prefixes for the human readable part
 #[derive(Eq, PartialEq, Debug, Clone, Copy)]
@@ -395,11 +385,6 @@ pub struct PayeePubKey(pub PublicKey);
 
 /// Positive duration that defines when (relatively to the timestamp) in the future the invoice
 /// expires
-///
-/// # Invariants
-/// The number of seconds this expiry time represents has to be in the range
-/// `0...(SYSTEM_TIME_MAX_UNIX_TIMESTAMP - MAX_EXPIRY_TIME)` to avoid overflows when adding it to a
-/// timestamp
 #[derive(Clone, Debug, Hash, Eq, PartialEq)]
 pub struct ExpiryTime(Duration);
 
@@ -459,11 +444,11 @@ impl InvoiceBuilder<tb::False, tb::False, tb::False, tb::False, tb::False> {
 			tagged_fields: Vec::new(),
 			error: None,
 
-			phantom_d: std::marker::PhantomData,
-			phantom_h: std::marker::PhantomData,
-			phantom_t: std::marker::PhantomData,
-			phantom_c: std::marker::PhantomData,
-			phantom_s: std::marker::PhantomData,
+			phantom_d: core::marker::PhantomData,
+			phantom_h: core::marker::PhantomData,
+			phantom_t: core::marker::PhantomData,
+			phantom_c: core::marker::PhantomData,
+			phantom_s: core::marker::PhantomData,
 		}
 	}
 }
@@ -479,11 +464,11 @@ impl<D: tb::Bool, H: tb::Bool, T: tb::Bool, C: tb::Bool, S: tb::Bool> InvoiceBui
 			tagged_fields: self.tagged_fields,
 			error: self.error,
 
-			phantom_d: std::marker::PhantomData,
-			phantom_h: std::marker::PhantomData,
-			phantom_t: std::marker::PhantomData,
-			phantom_c: std::marker::PhantomData,
-			phantom_s: std::marker::PhantomData,
+			phantom_d: core::marker::PhantomData,
+			phantom_h: core::marker::PhantomData,
+			phantom_t: core::marker::PhantomData,
+			phantom_c: core::marker::PhantomData,
+			phantom_s: core::marker::PhantomData,
 		}
 	}
 
@@ -507,10 +492,7 @@ impl<D: tb::Bool, H: tb::Bool, T: tb::Bool, C: tb::Bool, S: tb::Bool> InvoiceBui
 
 	/// Sets the expiry time
 	pub fn expiry_time(mut self, expiry_time: Duration) -> Self {
-        match ExpiryTime::from_duration(expiry_time) {
-            Ok(t) => self.tagged_fields.push(TaggedField::ExpiryTime(t)),
-            Err(e) => self.error = Some(e),
-        };
+		self.tagged_fields.push(TaggedField::ExpiryTime(ExpiryTime::from_duration(expiry_time)));
 		self
 	}
 
@@ -589,7 +571,8 @@ impl<D: tb::Bool, T: tb::Bool, C: tb::Bool, S: tb::Bool> InvoiceBuilder<D, tb::F
 }
 
 impl<D: tb::Bool, H: tb::Bool, C: tb::Bool, S: tb::Bool> InvoiceBuilder<D, H, tb::False, C, S> {
-	/// Sets the timestamp.
+	/// Sets the timestamp to a specific [`SystemTime`].
+	#[cfg(feature = "std")]
 	pub fn timestamp(mut self, time: SystemTime) -> InvoiceBuilder<D, H, tb::True, C, S> {
 		match PositiveTimestamp::from_system_time(time) {
 			Ok(t) => self.timestamp = Some(t),
@@ -599,7 +582,18 @@ impl<D: tb::Bool, H: tb::Bool, C: tb::Bool, S: tb::Bool> InvoiceBuilder<D, H, tb
 		self.set_flags()
 	}
 
-	/// Sets the timestamp to the current UNIX timestamp.
+	/// Sets the timestamp to a duration since the Unix epoch.
+	pub fn duration_since_epoch(mut self, time: Duration) -> InvoiceBuilder<D, H, tb::True, C, S> {
+		match PositiveTimestamp::from_duration_since_epoch(time) {
+			Ok(t) => self.timestamp = Some(t),
+			Err(e) => self.error = Some(e),
+		}
+
+		self.set_flags()
+	}
+
+	/// Sets the timestamp to the current system time.
+	#[cfg(feature = "std")]
 	pub fn current_timestamp(mut self) -> InvoiceBuilder<D, H, tb::True, C, S> {
 		let now = PositiveTimestamp::from_system_time(SystemTime::now());
 		self.timestamp = Some(now.expect("for the foreseeable future this shouldn't happen"));
@@ -776,7 +770,7 @@ impl SignedRawInvoice {
 macro_rules! find_extract {
 	($iter:expr, $enm:pat, $enm_var:ident) => {
 		find_all_extract!($iter, $enm, $enm_var).next()
-    };
+	};
 }
 
 /// Finds the all elements of an enum stream of a given variant and extracts one member of the
@@ -799,41 +793,18 @@ macro_rules! find_extract {
 /// ```
 macro_rules! find_all_extract {
 	($iter:expr, $enm:pat, $enm_var:ident) => {
-    	$iter.filter_map(|tf| match *tf {
+		$iter.filter_map(|tf| match *tf {
 			$enm => Some($enm_var),
 			_ => None,
 		})
-    };
+	};
 }
 
 #[allow(missing_docs)]
 impl RawInvoice {
-	/// Construct the invoice's HRP and signatureless data into a preimage to be hashed.
-	pub(crate) fn construct_invoice_preimage(hrp_bytes: &[u8], data_without_signature: &[u5]) -> Vec<u8> {
-		use bech32::FromBase32;
-
-		let mut preimage = Vec::<u8>::from(hrp_bytes);
-
-		let mut data_part = Vec::from(data_without_signature);
-		let overhang = (data_part.len() * 5) % 8;
-		if overhang > 0 {
-			// add padding if data does not end at a byte boundary
-			data_part.push(u5::try_from_u8(0).unwrap());
-
-			// if overhang is in (1..3) we need to add u5(0) padding two times
-			if overhang < 3 {
-				data_part.push(u5::try_from_u8(0).unwrap());
-			}
-		}
-
-		preimage.extend_from_slice(&Vec::<u8>::from_base32(&data_part)
-			.expect("No padding error may occur due to appended zero above."));
-		preimage
-	}
-
 	/// Hash the HRP as bytes and signatureless data part.
 	fn hash_from_parts(hrp_bytes: &[u8], data_without_signature: &[u5]) -> [u8; 32] {
-		let preimage = RawInvoice::construct_invoice_preimage(hrp_bytes, data_without_signature);
+		let preimage = construct_invoice_preimage(hrp_bytes, data_without_signature);
 		let mut hash: [u8; 32] = Default::default();
 		hash.copy_from_slice(&sha256::Hash::hash(&preimage)[..]);
 		hash
@@ -942,56 +913,57 @@ impl RawInvoice {
 }
 
 impl PositiveTimestamp {
-	/// Create a new `PositiveTimestamp` from a unix timestamp in the Range
-	/// `0...SYSTEM_TIME_MAX_UNIX_TIMESTAMP - MAX_EXPIRY_TIME`, otherwise return a
-	/// `CreationError::TimestampOutOfBounds`.
+	/// Creates a `PositiveTimestamp` from a Unix timestamp in the range `0..=MAX_TIMESTAMP`.
+	///
+	/// Otherwise, returns a [`CreationError::TimestampOutOfBounds`].
 	pub fn from_unix_timestamp(unix_seconds: u64) -> Result<Self, CreationError> {
-		if unix_seconds > SYSTEM_TIME_MAX_UNIX_TIMESTAMP - MAX_EXPIRY_TIME {
-			Err(CreationError::TimestampOutOfBounds)
-		} else {
-			Ok(PositiveTimestamp(UNIX_EPOCH + Duration::from_secs(unix_seconds)))
-		}
+		Self::from_duration_since_epoch(Duration::from_secs(unix_seconds))
 	}
 
-	/// Create a new `PositiveTimestamp` from a `SystemTime` with a corresponding unix timestamp in
-	/// the Range `0...SYSTEM_TIME_MAX_UNIX_TIMESTAMP - MAX_EXPIRY_TIME`, otherwise return a
-	/// `CreationError::TimestampOutOfBounds`.
+	/// Creates a `PositiveTimestamp` from a [`SystemTime`] with a corresponding Unix timestamp in
+	/// the range `0..=MAX_TIMESTAMP`.
+	///
+	/// Otherwise, returns a [`CreationError::TimestampOutOfBounds`].
+	#[cfg(feature = "std")]
 	pub fn from_system_time(time: SystemTime) -> Result<Self, CreationError> {
-		if time
-			.duration_since(UNIX_EPOCH)
-			.map(|t| t.as_secs() <= SYSTEM_TIME_MAX_UNIX_TIMESTAMP - MAX_EXPIRY_TIME)
-			.unwrap_or(true)
-			{
-				Ok(PositiveTimestamp(time))
-			} else {
+		time.duration_since(SystemTime::UNIX_EPOCH)
+			.map(Self::from_duration_since_epoch)
+			.unwrap_or(Err(CreationError::TimestampOutOfBounds))
+	}
+
+	/// Creates a `PositiveTimestamp` from a [`Duration`] since the Unix epoch in the range
+	/// `0..=MAX_TIMESTAMP`.
+	///
+	/// Otherwise, returns a [`CreationError::TimestampOutOfBounds`].
+	pub fn from_duration_since_epoch(duration: Duration) -> Result<Self, CreationError> {
+		if duration.as_secs() <= MAX_TIMESTAMP {
+			Ok(PositiveTimestamp(duration))
+		} else {
 			Err(CreationError::TimestampOutOfBounds)
 		}
 	}
 
-	/// Returns the UNIX timestamp representing the stored time
+	/// Returns the Unix timestamp representing the stored time
 	pub fn as_unix_timestamp(&self) -> u64 {
-		self.0.duration_since(UNIX_EPOCH)
-			.expect("ensured by type contract/constructors")
-			.as_secs()
+		self.0.as_secs()
 	}
 
-	/// Returns a reference to the internal `SystemTime` time representation
-	pub fn as_time(&self) -> &SystemTime {
-		&self.0
-	}
-}
-
-impl Into<SystemTime> for PositiveTimestamp {
-	fn into(self) -> SystemTime {
+	/// Returns the duration of the stored time since the Unix epoch
+	pub fn as_duration_since_epoch(&self) -> Duration {
 		self.0
 	}
+
+	/// Returns the [`SystemTime`] representing the stored time
+	#[cfg(feature = "std")]
+	pub fn as_time(&self) -> SystemTime {
+		SystemTime::UNIX_EPOCH + self.0
+	}
 }
 
-impl Deref for PositiveTimestamp {
-	type Target = SystemTime;
-
-	fn deref(&self) -> &Self::Target {
-		&self.0
+#[cfg(feature = "std")]
+impl Into<SystemTime> for PositiveTimestamp {
+	fn into(self) -> SystemTime {
+		SystemTime::UNIX_EPOCH + self.0
 	}
 }
 
@@ -1132,11 +1104,17 @@ impl Invoice {
 		Ok(invoice)
 	}
 
-	/// Returns the `Invoice`'s timestamp (should equal it's creation time)
-	pub fn timestamp(&self) -> &SystemTime {
+	/// Returns the `Invoice`'s timestamp (should equal its creation time)
+	#[cfg(feature = "std")]
+	pub fn timestamp(&self) -> SystemTime {
 		self.signed_invoice.raw_invoice().data.timestamp.as_time()
 	}
 
+	/// Returns the `Invoice`'s timestamp as a duration since the Unix epoch
+	pub fn duration_since_epoch(&self) -> Duration {
+		self.signed_invoice.raw_invoice().data.timestamp.0
+	}
+
 	/// Returns an iterator over all tagged fields of this Invoice.
 	///
 	/// (C-not exported) As there is not yet a manual mapping for a FilterMap
@@ -1190,11 +1168,13 @@ impl Invoice {
 	}
 
 	/// Returns whether the invoice has expired.
+	#[cfg(feature = "std")]
 	pub fn is_expired(&self) -> bool {
-		Self::is_expired_from_epoch(self.timestamp(), self.expiry_time())
+		Self::is_expired_from_epoch(&self.timestamp(), self.expiry_time())
 	}
 
 	/// Returns whether the expiry time from the given epoch has passed.
+	#[cfg(feature = "std")]
 	pub(crate) fn is_expired_from_epoch(epoch: &SystemTime, expiry_time: Duration) -> bool {
 		match epoch.elapsed() {
 			Ok(elapsed) => elapsed > expiry_time,
@@ -1202,6 +1182,14 @@ impl Invoice {
 		}
 	}
 
+	/// Returns whether the expiry time would pass at the given point in time.
+	/// `at_time` is the timestamp as a duration since the Unix epoch.
+	pub fn would_expire(&self, at_time: Duration) -> bool {
+		self.duration_since_epoch()
+			.checked_add(self.expiry_time())
+			.unwrap_or_else(|| Duration::new(u64::max_value(), 1_000_000_000 - 1)) < at_time
+	}
+
 	/// Returns the invoice's `min_final_cltv_expiry` time, if present, otherwise
 	/// [`DEFAULT_MIN_FINAL_CLTV_EXPIRY`].
 	pub fn min_final_cltv_expiry(&self) -> u64 {
@@ -1320,26 +1308,14 @@ impl Deref for PayeePubKey {
 }
 
 impl ExpiryTime {
-	/// Construct an `ExpiryTime` from seconds. If there exists a `PositiveTimestamp` which would
-	/// overflow on adding the `EpiryTime` to it then this function will return a
-	/// `CreationError::ExpiryTimeOutOfBounds`.
-	pub fn from_seconds(seconds: u64) -> Result<ExpiryTime, CreationError> {
-		if seconds <= MAX_EXPIRY_TIME {
-			Ok(ExpiryTime(Duration::from_secs(seconds)))
-		} else {
-			Err(CreationError::ExpiryTimeOutOfBounds)
-		}
+	/// Construct an `ExpiryTime` from seconds.
+	pub fn from_seconds(seconds: u64) -> ExpiryTime {
+		ExpiryTime(Duration::from_secs(seconds))
 	}
 
-	/// Construct an `ExpiryTime` from a `Duration`. If there exists a `PositiveTimestamp` which
-	/// would overflow on adding the `EpiryTime` to it then this function will return a
-	/// `CreationError::ExpiryTimeOutOfBounds`.
-	pub fn from_duration(duration: Duration) -> Result<ExpiryTime, CreationError> {
-		if duration.as_secs() <= MAX_EXPIRY_TIME {
-			Ok(ExpiryTime(duration))
-		} else {
-			Err(CreationError::ExpiryTimeOutOfBounds)
-		}
+	/// Construct an `ExpiryTime` from a `Duration`.
+	pub fn from_duration(duration: Duration) -> ExpiryTime {
+		ExpiryTime(duration)
 	}
 
 	/// Returns the expiry time in seconds
@@ -1408,14 +1384,17 @@ pub enum CreationError {
 	/// The specified route has too many hops and can't be encoded
 	RouteTooLong,
 
-	/// The unix timestamp of the supplied date is <0 or can't be represented as `SystemTime`
+	/// The Unix timestamp of the supplied date is less than zero or greater than 35-bits
 	TimestampOutOfBounds,
 
-	/// The supplied expiry time could cause an overflow if added to a `PositiveTimestamp`
-	ExpiryTimeOutOfBounds,
-
 	/// The supplied millisatoshi amount was greater than the total bitcoin supply.
 	InvalidAmount,
+
+	/// Route hints were required for this invoice and were missing. Applies to
+	/// [phantom invoices].
+	///
+	/// [phantom invoices]: crate::utils::create_phantom_invoice
+	MissingRouteHints,
 }
 
 impl Display for CreationError {
@@ -1423,13 +1402,14 @@ impl Display for CreationError {
 		match self {
 			CreationError::DescriptionTooLong => f.write_str("The supplied description string was longer than 639 bytes"),
 			CreationError::RouteTooLong => f.write_str("The specified route has too many hops and can't be encoded"),
-			CreationError::TimestampOutOfBounds => f.write_str("The unix timestamp of the supplied date is <0 or can't be represented as `SystemTime`"),
-			CreationError::ExpiryTimeOutOfBounds => f.write_str("The supplied expiry time could cause an overflow if added to a `PositiveTimestamp`"),
+			CreationError::TimestampOutOfBounds => f.write_str("The Unix timestamp of the supplied date is less than zero or greater than 35-bits"),
 			CreationError::InvalidAmount => f.write_str("The supplied millisatoshi amount was greater than the total bitcoin supply"),
+			CreationError::MissingRouteHints => f.write_str("The invoice required route hints and they weren't provided"),
 		}
 	}
 }
 
+#[cfg(feature = "std")]
 impl std::error::Error for CreationError { }
 
 /// Errors that may occur when converting a `RawInvoice` to an `Invoice`. They relate to the
@@ -1485,6 +1465,7 @@ impl Display for SemanticError {
 	}
 }
 
+#[cfg(feature = "std")]
 impl std::error::Error for SemanticError { }
 
 /// When signing using a fallible method either an user-supplied `SignError` or a `CreationError`
@@ -1514,17 +1495,10 @@ mod test {
 
 	#[test]
 	fn test_system_time_bounds_assumptions() {
-		::check_platform();
-
-        assert_eq!(
-            ::PositiveTimestamp::from_unix_timestamp(::SYSTEM_TIME_MAX_UNIX_TIMESTAMP + 1),
-            Err(::CreationError::TimestampOutOfBounds)
-        );
-
-        assert_eq!(
-            ::ExpiryTime::from_seconds(::MAX_EXPIRY_TIME + 1),
-            Err(::CreationError::ExpiryTimeOutOfBounds)
-        );
+		assert_eq!(
+			::PositiveTimestamp::from_unix_timestamp(::MAX_TIMESTAMP + 1),
+			Err(::CreationError::TimestampOutOfBounds)
+		);
 	}
 
 	#[test]
@@ -1727,7 +1701,7 @@ mod test {
 		let builder = InvoiceBuilder::new(Currency::Bitcoin)
 			.description("Test".into())
 			.payment_hash(sha256::Hash::from_slice(&[0;32][..]).unwrap())
-			.current_timestamp();
+			.duration_since_epoch(Duration::from_secs(1234567));
 
 		let invoice = builder.clone()
 			.amount_milli_satoshis(1500)
@@ -1756,7 +1730,7 @@ mod test {
 
 		let builder = InvoiceBuilder::new(Currency::Bitcoin)
 			.payment_hash(sha256::Hash::from_slice(&[0;32][..]).unwrap())
-			.current_timestamp()
+			.duration_since_epoch(Duration::from_secs(1234567))
 			.min_final_cltv_expiry(144);
 
 		let too_long_string = String::from_iter(
@@ -1872,7 +1846,7 @@ mod test {
 
 		let builder = InvoiceBuilder::new(Currency::BitcoinTestnet)
 			.amount_milli_satoshis(123)
-			.timestamp(UNIX_EPOCH + Duration::from_secs(1234567))
+			.duration_since_epoch(Duration::from_secs(1234567))
 			.payee_pub_key(public_key.clone())
 			.expiry_time(Duration::from_secs(54321))
 			.min_final_cltv_expiry(144)
@@ -1894,6 +1868,7 @@ mod test {
 		assert_eq!(invoice.amount_milli_satoshis(), Some(123));
 		assert_eq!(invoice.amount_pico_btc(), Some(1230));
 		assert_eq!(invoice.currency(), Currency::BitcoinTestnet);
+		#[cfg(feature = "std")]
 		assert_eq!(
 			invoice.timestamp().duration_since(UNIX_EPOCH).unwrap().as_secs(),
 			1234567
@@ -1925,7 +1900,7 @@ mod test {
 			.description("Test".into())
 			.payment_hash(sha256::Hash::from_slice(&[0;32][..]).unwrap())
 			.payment_secret(PaymentSecret([0; 32]))
-			.current_timestamp()
+			.duration_since_epoch(Duration::from_secs(1234567))
 			.build_raw()
 			.unwrap()
 			.sign::<_, ()>(|hash| {
@@ -1938,7 +1913,7 @@ mod test {
 
 		assert_eq!(invoice.min_final_cltv_expiry(), DEFAULT_MIN_FINAL_CLTV_EXPIRY);
 		assert_eq!(invoice.expiry_time(), Duration::from_secs(DEFAULT_EXPIRY_TIME));
-		assert!(!invoice.is_expired());
+		assert!(!invoice.would_expire(Duration::from_secs(1234568)));
 	}
 
 	#[test]
@@ -1947,14 +1922,11 @@ mod test {
 		use secp256k1::Secp256k1;
 		use secp256k1::key::SecretKey;
 
-		let timestamp = SystemTime::now()
-			.checked_sub(Duration::from_secs(DEFAULT_EXPIRY_TIME * 2))
-			.unwrap();
 		let signed_invoice = InvoiceBuilder::new(Currency::Bitcoin)
 			.description("Test".into())
 			.payment_hash(sha256::Hash::from_slice(&[0;32][..]).unwrap())
 			.payment_secret(PaymentSecret([0; 32]))
-			.timestamp(timestamp)
+			.duration_since_epoch(Duration::from_secs(1234567))
 			.build_raw()
 			.unwrap()
 			.sign::<_, ()>(|hash| {
@@ -1965,6 +1937,6 @@ mod test {
 			.unwrap();
 		let invoice = Invoice::from_signed(signed_invoice).unwrap();
 
-		assert!(invoice.is_expired());
+		assert!(invoice.would_expire(Duration::from_secs(1234567 + DEFAULT_EXPIRY_TIME + 1)));
 	}
 }
